#include <streamer/streamer.h>
#include <fab_ppm.h>
#include "common.h"

StreamElementType *_fab_ppm;

StreamElement *Stream_fab_ppm_create() {
  Stream_fab_ppm *element;

  element = (Stream_fab_ppm *)malloc(sizeof(Stream_fab_ppm));

  element->index = 0;

  element->element.flags = 0;
  element->element.inpadfunc = NULL; /* leave it up to general case */
  element->element.outpadfunc = NULL; /* ditto */
  element->element.chain = NULL; /* none */
  element->element.push = Stream_fab_ppm_push;
  element->element.inpads = NULL;
  element->element.outpads = NULL;

  element->out.type = 0;
  element->out.flags = 0;
  element->out.owner = (StreamElement *)element;
  element->out.peer = NULL;
  element->out.chain = NULL;
  /* register it */
  element->element.outpads =
    g_slist_append(element->element.outpads,&element->out);

  return (StreamElement *)element;
}

void Stream_fab_ppm_setsize(StreamElement *element,int w,int h) {
  Stream_fab_ppm *fab_ppm_element = (Stream_fab_ppm *)element;

  fab_ppm_element->w = w;
  fab_ppm_element->h = h;
  fab_ppm_element->imagesize = w*h*3;

  if (!fab_ppm_element->pool)
    fab_ppm_element->pool = StreamBufferPool_create(8,0,0,w*h*3);
}

void Stream_fab_ppm_push(StreamElement *element) {
  Stream_fab_ppm *fab_ppm_element = (Stream_fab_ppm *)element;
  StreamBuffer *buf;

  if (fab_ppm_element->out.chain) {
    buf = StreamBuffer_create(fab_ppm_element->pool);
    sprintf(buf->data,"%d\n",fab_ppm_element->index++);
    DEBUG("fab_ppm created buffer %p\n",buf);

    DEBUG("fab_ppm about to chain to %p()\n",fab_ppm_element->out.chain);
    StreamPad_chain(&fab_ppm_element->out,buf);
  }
}


void init_StreamPlugin(StreamPlugin *plugin) {
  /* we have to do this because we're going to return static data */
  g_module_make_resident(plugin->module);

  /* fill in the StreamPlugin data */
  plugin->name = "fab_ppm";
  plugin->longname = "Fabrication filters";
  plugin->version = "0.0.0";
  plugin->elementtypes = NULL;
  plugin->enumtypes = NULL; /* simple case, we rely on default */

  /* create the element types */
  _fab_ppm = (StreamElementType *)g_malloc(sizeof(StreamElementType));
  _fab_ppm->plugin = plugin;
  _fab_ppm->name = "fab_ppm";
  _fab_ppm->longname = "Fabrication";
  _fab_ppm->version = "0.0.0";
  _fab_ppm->create = Stream_fab_ppm_create;

  /* add it to necessary lists */
  plugin->elementtypes = g_slist_append(plugin->elementtypes,_fab_ppm);
  StreamElementType_register(_fab_ppm);

  /* announce that we're here... */
  DEBUG("initialized plugin '%s'\n",StreamPlugin_getname(plugin));
}

