/* Copyright (C) 1998 Oregon Graduate Institute of Science & Technology
 * 
 * See the file COPYRIGHT, which should have been supplied with this
 * package, for licensing details.  We may be contacted through email
 * at <quasar-help@cse.ogi.edu>.
 */

#ifdef HAVE_LAUNCHER
#include "init_modules.h"
#else
#include "global.h"
#endif /* HAVE_LAUNCHER */

#include <string.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <netinet/in.h>
#include <sys/time.h>

#include <dbug/dbug.h>
#include "mkindex.h"
#include "stream.h"
#include "channel.h"
#include "packet.h"

#include <pbm.h>
#include "pbmfont.h"

void printtoimage(struct font *fn,unsigned char *string,int x,int y,
                  unsigned char *framebuf,int width,int height) {
  struct glyph *glyph;
  int l,i,w,gx,gy;

  /* start with frame number */
  l = strlen(string);
  for (i=0;i<l;i++) {
    if (!(glyph = fn->glyph[string[i]]))
      glyph = fn->glyph[' '];
    w += glyph->xadd;
  }
  if ((x+w) > width) x = width - w;
  for (i=0;i<l;i++) {
    if (!(glyph = fn->glyph[string[i]]))
      glyph = fn->glyph[' '];
    for (gy=0;gy<glyph->height;gy++) {
      int goff = gy*glyph->width;
      int ioff = (y+gy)*(width*3);
      for (gx=0;gx<glyph->width;gx++) {
        if (glyph->bmap[goff+gx]) {
          *(unsigned char *)(framebuf + ioff + ((x+gx)*3)) = 255;
          *(unsigned char *)(framebuf + ioff + ((x+gx)*3+1)) = 255;
          *(unsigned char *)(framebuf + ioff + ((x+gx)*3+2)) = 255;
        }
      }
    }
    x += glyph->xadd;
  }
}


#ifdef HAVE_LAUNCHER
void
ppm_fab(int argc, char **argv) {
  int width = 160,height = 120;		/* width/height of capture */

  channel_dst *outpacket = NULL;
  unsigned char ppmhead[32];
  int ppmheadlen;
  unsigned char *framebuf;
  int seq = 0;
  struct font *fn;

  int done = 0;

  DBUG_ENTER();

  argc = module_init(argc, argv, NULL, ARG_null, &outpacket, ARG_packet);

  pthread_mutex_lock(&launcher_mutex);

  while (!done) {
    int c = getopt(argc, argv, "o:x:y:");

    switch (c) {
      case 'o':
        outpacket = open_channel_dst(optarg, ARG_packet);
        break;
      case 'x':
        width = atoi(optarg);
        break;
      case 'y':
        height = atoi(optarg);
        break;
      default:
        printf("%s: Unkown option\n",argv[0]);
        exit(1);
      case EOF:
        done = 1;;
    }
  }

  done = optind;
  optind = 0;  // to "reset" getopt!!!
  pthread_mutex_unlock(&launcher_mutex);

  if ((argc - done) == 1) {
    outpacket = open_channel_dst(argv[done], ARG_packet);
    argc--;
  }

  if ((outpacket == NULL)) {
    printf(" Something's wrong with %s's args\n",argv[0]);
    exit(1);
  }

  ppmheadlen = sprintf(ppmhead,"P6\n%d %d 255\n", width, height);
  fprintf(stderr,"creating image of %dx%d\n",width,height);
  fn = pbm_defaultfont("fixed");

  for (;;) {
    dbuf *frame;
    pack_head *fhead;
    unsigned char string[64];
    struct timeval tstamp;

    /* allocate the dbuf and get a pointer to the data area */
    frame = new_raw_dbuf(ppmheadlen);
    dbGROW(frame, sizeof(pack_head) + ppmheadlen + width*height*3);
    framebuf = (unsigned char *)dbCUR(frame);

    /* create the packet header */
    fhead = (pack_head *)framebuf;
    fhead->len = htons(sizeof(pack_head) + ppmheadlen + width*height*3);
    fhead->seq = seq++;
    fhead->flags = 0;
    framebuf += sizeof(pack_head);

    /* put the PPM header in */
    memcpy(framebuf,ppmhead,ppmheadlen);
    framebuf += ppmheadlen;

    /* now go create the image */
    memset(framebuf,0,width*height*3);
    /* try to print stuff */
    sprintf(string,"Frame: %5d",seq);
    printtoimage(fn,string,5,5,framebuf,width,height);
    gettimeofday(&tstamp,NULL);
    sprintf(string,"Time: %d.%06d",tstamp.tv_sec,tstamp.tv_usec);
    printtoimage(fn,string,5,20,framebuf,width,height);

    /* send it off to the next pipeline element */
    chSEND_free(outpacket, frame);
  }
  DBUG_VOID_RETURN();
}
#endif /* HAVE_LAUNCHER */
